#!/usr/bin/env python3

# (c) Copyright 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors ('Micro Focus') are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be liable
# for technical or editorial errors or omissions contained herein. The information
# contained herein is subject to change without notice.

"""
Python script to get the health status of novell-cifs service
"""

import fcntl
import os.path
import subprocess
import json
import syslog
from collections import OrderedDict
from time import strptime, mktime

cifs_config_file = "/etc/opt/novell/cifs/cifs.conf"
cifs_jsonfile = "/opt/novell/cifs/novell-cifs_Health.json"

nit_HealthScript = "/opt/novell/dashboard/bin/nithealth.sh"
nit_jsonfile = "/var/opt/novell/nit/novell-nit.json"

ncp_ndsd_HealthScript = "/opt/novell/dashboard/bin/NCPandNDSHealth.sh"
ncp_ndsd_jsonfile = "/var/opt/novell/ncp/ncpserv.json"

nss_HealthScript = "/opt/novell/dashboard/bin/nss_health"

healthy		= "Healthy"
unhealthy	= "Unhealthy"


"""
Write novell-cifs health data to '/opt/novell/cifs/novell-cifs_Health.json'
"""
def write_to_jsonfile(healthData):

	try:
		with open(cifs_jsonfile, "w") as fd:
			fcntl.flock(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
			json.dump(healthData, fd, indent=4, separators=(", ", ": "))
			fcntl.flock(fd, fcntl.LOCK_UN)
	except IOError:
		syslog.syslog(syslog.LOG_INFO, "Failed to open/write json file: " + cifs_jsonfile)


"""
Check if CIFSD is listening on specified ports 139 & 445
"""
def is_cifsd_listening_on_ports_139_and_445():

	retcode, listening_on_port = subprocess.getstatusoutput("netstat -ltnp | grep ':139' | grep cifsd ")
	if ((retcode == 0) and (listening_on_port != "")):
		retcode, listening_on_port = subprocess.getstatusoutput("netstat -ltnp | grep ':445' | grep cifsd ")
		if ((retcode == 0) and (listening_on_port != "")):
			return True
		else:
			return False
	else:
		return False


"""
Fill nit health data
"""
def fill_nit_healthdata(novellcifs_health_jsonstream):

	if os.path.exists(nit_HealthScript):
		nit_healthdata = subprocess.getoutput(nit_HealthScript)

		if os.path.exists(nit_jsonfile):
			nit_healthdata = json.loads(nit_healthdata)
			novellcifs_health_jsonstream["children"][0]["serviceName"] = nit_healthdata["serviceName"]
			novellcifs_health_jsonstream["children"][0]["status"] = nit_healthdata["status"]

			if ("runningSince" in nit_healthdata) and (nit_healthdata["runningSince"] != 0):
				novellcifs_health_jsonstream["children"][0]["runningSince"] = nit_healthdata["runningSince"]
			else:
				del novellcifs_health_jsonstream["children"][0]["runningSince"]

			if ("AdditionalInfo" in nit_healthdata) and (nit_healthdata["AdditionalInfo"] != ""):
				novellcifs_health_jsonstream["children"][0]["AdditionalInfo"] = nit_healthdata["AdditionalInfo"]
			else:
				del novellcifs_health_jsonstream["children"][0]["AdditionalInfo"]

			return 0
		else:
			return 1
	else:
		return 1


"""
Fill eDir health data
"""
def fill_edir_healthdata(novellcifs_health_jsonstream):

	if os.path.exists(ncp_ndsd_HealthScript):
		eDir_healthdata = subprocess.getoutput(ncp_ndsd_HealthScript)

		if os.path.exists(ncp_ndsd_jsonfile):
			eDir_healthdata = json.loads(eDir_healthdata)
			novellcifs_health_jsonstream["children"][1]["serviceName"] = eDir_healthdata["children"][1]["serviceName"]
			novellcifs_health_jsonstream["children"][1]["status"] = eDir_healthdata["children"][1]["status"]

			if ("runningSince" in eDir_healthdata["children"][1]) and (eDir_healthdata["children"][1]["runningSince"] != 0):
				novellcifs_health_jsonstream["children"][1]["runningSince"] = eDir_healthdata["children"][1]["runningSince"]
			else:
				del novellcifs_health_jsonstream["children"][1]["runningSince"]

			if ("AdditionalInfo" in eDir_healthdata["children"][1]) and (eDir_healthdata["children"][1]["AdditionalInfo"] != ""):
				novellcifs_health_jsonstream["children"][1]["AdditionalInfo"] = eDir_healthdata["children"][1]["AdditionalInfo"]
			else:
				del novellcifs_health_jsonstream["children"][1]["AdditionalInfo"]

			return 0
		else:
			return 1
	else:
		return 1


"""
Fill nss health data
"""
def fill_nss_healthdata(novellcifs_health_jsonstream):

	if os.path.exists(nss_HealthScript):
		retcode, nss_healthdata = subprocess.getstatusoutput(nss_HealthScript)
		if (retcode != 0):
			return 1
		else:
			nss_healthdata = json.loads(nss_healthdata)
			novellcifs_health_jsonstream["children"][2]["serviceName"] = nss_healthdata["serviceName"]
			novellcifs_health_jsonstream["children"][2]["status"] = nss_healthdata["status"]

			if ("runningSince" in nss_healthdata) and (nss_healthdata["runningSince"] != 0):
				novellcifs_health_jsonstream["children"][2]["runningSince"] = nss_healthdata["runningSince"]
			else:
				del novellcifs_health_jsonstream["children"][2]["runningSince"]

			if ("AdditionalInfo" in nss_healthdata) and (nss_healthdata["AdditionalInfo"] != ""):
				novellcifs_health_jsonstream["children"][2]["AdditionalInfo"] = nss_healthdata["AdditionalInfo"]
			else:
				del novellcifs_health_jsonstream["children"][2]["AdditionalInfo"]

			return 0
	else:
		return 1


"""
Get novell-cifs Health data
"""
def get_novellcifs_HealthData():

	novellcifs_health_jsonstream = OrderedDict(
					[
						("serviceName", "OES CIFS"),
						("status", ""),
						("runningSince", 0),
						("AdditionalInfo", ""),
						("children",
							[
								OrderedDict(
								[
									# To store nit health data
									("serviceName", ""),
									("status", ""),
									("runningSince", 0),
									("AdditionalInfo", "")
								]),
								OrderedDict(
								[
									# To store eDirectory health data
									("serviceName", ""),
									("status", ""),
									("runningSince", 0),
									("AdditionalInfo", "")
								]),
								OrderedDict(
								[
									# To store NSS health data
									("serviceName", ""),
									("status", ""),
									("runningSince", 0),
									("AdditionalInfo", "")
								])
							]
						)
					])

	novellcifs_systemctl_status_cmd = "systemctl is-active novell-cifs.service"
	retcode, novellcifs_status = subprocess.getstatusoutput(novellcifs_systemctl_status_cmd)
	if ((retcode != 0) and (novellcifs_status != "inactive")):
		syslog.syslog("Error in getting novell-cifs.service status")
		exit(1)

	retcode, novcifs_o_status = subprocess.getstatusoutput('novcifs -o')
	if (retcode == 0):
		novcifs_o_status = "SUCCESS"
	else:
		novcifs_o_status = "FAIL"


	cifsd_listening_on_ports_139_and_445 = is_cifsd_listening_on_ports_139_and_445()


	if (fill_nit_healthdata(novellcifs_health_jsonstream) != 0):
		syslog.syslog("Error in getting novell-nit.service health data")
		exit(1)
	else:
		nit_healthstatus = novellcifs_health_jsonstream["children"][0]["status"]


	if (fill_edir_healthdata(novellcifs_health_jsonstream) != 0):
		syslog.syslog("Error in getting eDirectory health data")
		exit(1)
	else:
		eDir_healthstatus = novellcifs_health_jsonstream["children"][1]["status"]


	if (fill_nss_healthdata(novellcifs_health_jsonstream) != 0):
		syslog.syslog("Error in getting novell-nss.service health data")
		exit(1)
	else:
		nss_healthstatus = novellcifs_health_jsonstream["children"][2]["status"]


	if ((novellcifs_status == "active") and (nit_healthstatus == healthy) and
			(eDir_healthstatus == healthy) and
			(nss_healthstatus == healthy) and
			(novcifs_o_status == "SUCCESS") and
			(cifsd_listening_on_ports_139_and_445 == True) and
			(os.path.exists(cifs_config_file) == True)):

		novellcifs_health_jsonstream["status"] = healthy

		timeStamp_cmd = "systemctl show -p WatchdogTimestamp novell-cifs.service"
		retcode, timeStamp = subprocess.getstatusoutput(timeStamp_cmd)
		if (retcode == 0):
			novellcifs_runningSince_Info = timeStamp.strip().split("=")
			epochTimeStamp = mktime(strptime(novellcifs_runningSince_Info[1], "%a %Y-%m-%d %H:%M:%S %Z"))
			novellcifs_health_jsonstream["runningSince"] = int(epochTimeStamp)
		else:
			del novellcifs_health_jsonstream["runningSince"]

		del novellcifs_health_jsonstream["AdditionalInfo"]

	else:

		novellcifs_health_jsonstream["status"] = unhealthy
		del novellcifs_health_jsonstream["runningSince"]

		if (novellcifs_status == "inactive"):
			novellcifs_health_jsonstream["AdditionalInfo"] = ["CIFS ", "Common_ServiceInactive", "Common_Resolve", " novell-cifs.service."]
		elif (cifsd_listening_on_ports_139_and_445 == False):
			novellcifs_health_jsonstream["AdditionalInfo"] = ["CIFS_INFO_NotListeningOnPorts", "Common_ResolveService"]
		elif (os.path.exists(cifs_config_file) == False):
			novellcifs_health_jsonstream["AdditionalInfo"] = "CIFS_INFO_NoConfigFile"
		elif (novcifs_o_status == "FAIL"):
			novellcifs_health_jsonstream["AdditionalInfo"] = ["CIFS_INFO_CommunicationBroken", "Common_ResolveService"]
		else:
			novellcifs_health_jsonstream["AdditionalInfo"] = ["Common_DependentService", " novell-cifs.service ", "Common_Inactives", "Common_ResolveServices"]

	return novellcifs_health_jsonstream
"""
End of get_novellcifs_HealthData()
"""


"""
script starts from here
"""
if os.path.exists(cifs_jsonfile):
	os.remove(cifs_jsonfile)

retVal, service_config_status = subprocess.getstatusoutput("systemctl is-enabled novell-cifs.service")
if ((retVal == 0) and (service_config_status == "enabled")):
	novellcifs_HealthData = get_novellcifs_HealthData()
	print(json.dumps(novellcifs_HealthData, sort_keys=False, indent=4))
	write_to_jsonfile(novellcifs_HealthData)
else:
	syslog.syslog("novell-cifs.service may not have been configured")

exit(0)

